/*
  USB_keyboard_mouse
  Ejemplo simple de USB nativo en el Arduino Due
*/

#include "keycodes.h"

#define M(n) ((n) & 0x3FFF)

char keycodes_ascii[] = {
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, M(KEY_ENTER), 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
M(ASCII_20), M(ASCII_21), M(ASCII_22), M(ASCII_23),
M(ASCII_24), M(ASCII_25), M(ASCII_26), M(ASCII_27),
M(ASCII_28), M(ASCII_29), M(ASCII_2A), M(ASCII_2B),
M(ASCII_2C), M(ASCII_2D), M(ASCII_2E), M(ASCII_2F),
M(ASCII_30), M(ASCII_31), M(ASCII_32), M(ASCII_33),
M(ASCII_34), M(ASCII_35), M(ASCII_36), M(ASCII_37),
M(ASCII_38), M(ASCII_39), M(ASCII_3A), M(ASCII_3B),
M(ASCII_3C), M(ASCII_3D), M(ASCII_3E), M(ASCII_3F),
M(ASCII_40), M(ASCII_41), M(ASCII_42), M(ASCII_43),
M(ASCII_44), M(ASCII_45), M(ASCII_46), M(ASCII_47),
M(ASCII_48), M(ASCII_49), M(ASCII_4A), M(ASCII_4B),
M(ASCII_4C), M(ASCII_4D), M(ASCII_4E), M(ASCII_4F),
M(ASCII_50), M(ASCII_51), M(ASCII_52), M(ASCII_53),
M(ASCII_54), M(ASCII_55), M(ASCII_56), M(ASCII_57),
M(ASCII_58), M(ASCII_59), M(ASCII_5A), M(ASCII_5B),
M(ASCII_5C), M(ASCII_5D), M(ASCII_5E), M(ASCII_5F),
M(ASCII_60), M(ASCII_61), M(ASCII_62), M(ASCII_63),
M(ASCII_64), M(ASCII_65), M(ASCII_66), M(ASCII_67),
M(ASCII_68), M(ASCII_69), M(ASCII_6A), M(ASCII_6B),
M(ASCII_6C), M(ASCII_6D), M(ASCII_6E), M(ASCII_6F),
M(ASCII_70), M(ASCII_71), M(ASCII_72), M(ASCII_73),
M(ASCII_74), M(ASCII_75), M(ASCII_76), M(ASCII_77),
M(ASCII_78), M(ASCII_79), M(ASCII_7A), M(ASCII_7B),
M(ASCII_7C), M(ASCII_7D), M(ASCII_7E), M(ASCII_7F)
};

/*
  tecPrint
    Simula la pulsación de las teclas correspondientes a los caracteres de la cadena que
    se le pasa como argumento.
  Parámetros de entrada:
    char* cadena: cadena que se quiere teclear.
  Devuelve:
    Nada.
*/
void tecPrint(char *cadena) {
  char c;                                // Carácter a imprimir
  while(c = *cadena++) {                 // Recorre la cadena pasada como argumento
    if (!c) return;                      // Si el carácter es el cero, regresa
    c = keycodes_ascii[c];               // Convierte carácter a keycode
    if (c & SHIFT_MASK) {                // Si es una mayúscula o cualquier tecla con SHIFT
      Keyboard.press(M(KEY_LEFT_SHIFT)); // Presiona el SHIFT izquierdo
      c &= ~SHIFT_MASK;                  //  y elimina el SHIFT del keycode
    }
    Keyboard.press(c + 136);             // Simula pulsación tecla
    delay(20);                           // Espera 20 ms
    Keyboard.releaseAll();               // Suelta todas las teclas
  }
}

#define RPIN  8          // Pin donde está conectado el LED rojo
#define GPIN  7          // Pin donde está conectado el LED verde
#define BPIN  6          // Pin donde está conectado el LED azul
#define PULSPIN  13      // Pin donde está conectado el pulsador
#define ST_MOUSE  0      // Indicador de simulación de ratón
#define ST_KB  1         // Indicador de simulación de teclado

#define RADX  250.0      // Radio mayor de la elipse a describir con el puntero del ratón
#define RADY  150.0      // Radio menor de la elipse a describir con el puntero del ratón

float  x = RADX, y = 0,  // Coordenadas actuales del puntero del ratón
  xa, ya,                // Coordenadas anteriores del puntero del ratón
  grados = 5;            // Ángulo del próximo punto donde ubicar el puntero del ratón
int  incrx, incry,       // Incremento a mover el ratón
  estado;                // Indica el estado actual: simulando ratón o teclado

// Función que activa el LED RGB con una de las 8 combinaciones de color básicas
void ledRGB(int r, int g, int b) {
  digitalWrite(RPIN, r == 0);
  digitalWrite(GPIN, g == 0);
  digitalWrite(BPIN, b == 0);
}


// Setup
// -----
void setup() {
  // Configura los pines de E/S del pulsador y el LED
  pinMode(PULSPIN, INPUT_PULLUP);  // Configura pin pulsador como entrada con pull-up     
  pinMode(RPIN, OUTPUT);           // Configura pin LED rojo como salida
  pinMode(GPIN, OUTPUT);           // Configura pin LED verde como salida
  pinMode(BPIN, OUTPUT);           // Configura pin LED azul como salida
  
  ledRGB(0, 0, 0);                 // Apaga el LED
  estado = ST_MOUSE;               // Inicia simulación con ratón
  Mouse.begin();                   // Inicializa ratón
  Keyboard.begin();                // Inicializa teclado
}


// Loop
// ----
void loop() {
  if (estado == ST_MOUSE)          // Se comporta como un ratón USB
  { 
    ledRGB(0, 0, 1);               // Enciende el LED azul
    // ************* MOVIMIENTO EN CÍRCULO DEL PUNTERO DEL RATÓN ***************
    xa = x;                        // Salvaguarda coordenada x del punto anterior
    ya = y;                        // Salvaguarda cordenada y del punto anterior
    x = RADX * sin(grados/52.7);   // Calcula coordenada x del punto actual
    y = RADY * cos(grados/52.7);   // Calcula coordenada y del punto actual
    incrx = x - xa;                // Calcula incremento movimiento eje x
    incry = y - ya;                // Calcula incremento movimiento eje y
    Mouse.move(incrx, incry);      // Realiza movimiento ratón
    grados += 5;                   // Incrementa ángulo
    // *************************************************************************
    delay(50);                     // Introduce un retardo de 50 ms
    // Si se presiona el pulsador, pasa a simular un teclado
    if (digitalRead(PULSPIN) == LOW) {
      ledRGB(1, 0, 0);                     // Enciende el LED rojo
      while(digitalRead(PULSPIN) == LOW);  // Espera a que se suelte el pulsador
      delay(100);                  // Introduce un retardo de 100 ms para evitar rebotes
      estado = ST_KB;              // Cambia el estado al modo teclado
      Mouse.end();                 // Finaliza el comportamiento como ratón
    }
  }
  else if (estado == ST_KB)        // Se comporta como un teclado USB
  {
    ledRGB(0, 1, 0);                     // Enciende el LED en color verde
    while(digitalRead(PULSPIN) == HIGH); // Espera a que se presione el pulsador
    ledRGB(1, 0, 1);                     // Enciende el LED en color violeta (rojo+azul)
    // Escribe el texto en la ventana activa
    tecPrint("\n\n");
    tecPrint("Universitat Jaume I - EI1004/MT1004\n");
    delay(1000);
    tecPrint("Mas experiencias como esta en la intensificacion\n");
    delay(1000);
    tecPrint("\n   ---===I INGENIERIA DE COMPUTADORES I===---\n\n");
    delay(1000);           // Espera 1 segundo
    Keyboard.end();        // Finaliza el comportamiento como teclado
    while(1);              // Bucle infinito
  }
}

